@@ -1,7 +1,7 @@
 /* Lepton EDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
  * Copyright (C) 1998-2014 gEDA Contributors
- * Copyright (C) 2017-2020 Lepton EDA Contributors
+ * Copyright (C) 2017-2021 Lepton EDA Contributors
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,7 +93,10 @@ static void
 update_magnetic_net_label (GschemBottomWidget *widget, GParamSpec *pspec, gpointer unused);
 
 
-static GObjectClass *gschem_bottom_widget_parent_class = NULL;
+
+/* convenience macro - gobject type implementation:
+*/
+G_DEFINE_TYPE (GschemBottomWidget, gschem_bottom_widget, GTK_TYPE_HBOX);
 
 
 
@@ -104,8 +107,10 @@ dispose (GObject *object)
 {
   /* lastly, chain up to the parent dispose */
 
-  g_return_if_fail (gschem_bottom_widget_parent_class != NULL);
-  gschem_bottom_widget_parent_class->dispose (object);
+  GschemBottomWidgetClass* cls = GSCHEM_BOTTOM_WIDGET_GET_CLASS (object);
+  GObjectClass* parent_cls = (GObjectClass*) g_type_class_peek_parent (cls);
+
+  parent_cls->dispose (object);
 }
 
 
@@ -117,8 +122,10 @@ finalize (GObject *object)
 {
   /* lastly, chain up to the parent finalize */
 
-  g_return_if_fail (gschem_bottom_widget_parent_class != NULL);
-  gschem_bottom_widget_parent_class->finalize (object);
+  GschemBottomWidgetClass* cls = GSCHEM_BOTTOM_WIDGET_GET_CLASS (object);
+  GObjectClass* parent_cls = (GObjectClass*) g_type_class_peek_parent (cls);
+
+  parent_cls->finalize (object);
 }
 
 
@@ -190,8 +197,6 @@ get_property (GObject *object, guint param_id, GValue *value, GParamSpec *pspec)
 static void
 gschem_bottom_widget_class_init (GschemBottomWidgetClass *klass)
 {
-  gschem_bottom_widget_parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-
   G_OBJECT_CLASS (klass)->dispose  = dispose;
   G_OBJECT_CLASS (klass)->finalize = finalize;
 
@@ -459,38 +464,261 @@ gschem_bottom_widget_get_magnetic_net_mode (GschemBottomWidget *widget)
 
 
 
-/*! \brief Get/register GschemBottomWidget type.
+/*! \brief Set grid_snap_widget text
+ *
+ *  \param  widget  This GschemBottomWidget
+ *  \param  str     Text to set. Newly-allocated string. Will be g_free()'d
  */
-GType
-gschem_bottom_widget_get_type ()
+static void
+set_snap_info_widget_text (GschemBottomWidget* widget,
+                           gchar* str)
 {
-  static GType type = 0;
+  g_return_if_fail (widget != NULL);
 
-  if (type == 0) {
-    static const GTypeInfo info = {
-      sizeof(GschemBottomWidgetClass),
-      NULL,                                                    /* base_init */
-      NULL,                                                    /* base_finalize */
-      (GClassInitFunc) gschem_bottom_widget_class_init,
-      NULL,                                                    /* class_finalize */
-      NULL,                                                    /* class_data */
-      sizeof(GschemBottomWidget),
-      0,                                                       /* n_preallocs */
-      (GInstanceInitFunc) gschem_bottom_widget_init,
-    };
+  gtk_label_set_markup (GTK_LABEL(widget->grid_snap_widget), str);
+  g_free (str);
+}
+
+
+
+/*! \brief Set grid_snap_widget color
+ *
+ *  \param  widget      This GschemBottomWidget
+ *  \param  color_name  Color name, e.g. "red", "blue"
+ */
+static void
+set_snap_info_widget_color (GschemBottomWidget* widget,
+                            const gchar* color_name)
+{
+  g_return_if_fail (widget != NULL);
 
-    type = g_type_register_static (
 #ifdef ENABLE_GTK3
-                                   GTK_TYPE_BOX,
+  GdkRGBA color;
+  gdk_rgba_parse (&color, color_name);
+  gtk_widget_override_color (GTK_WIDGET (widget->grid_snap_widget),
+                             GTK_STATE_FLAG_NORMAL,
+                             &color);
 #else
-                                   GTK_TYPE_HBOX,
+  GdkColor color;
+  gdk_color_parse (color_name, &color);
+  gtk_widget_modify_fg (GTK_WIDGET (widget->grid_snap_widget),
+                        GTK_STATE_NORMAL,
+                        &color);
 #endif
-                                   "GschemBottomWidget",
-                                   &info,
-                                   (GTypeFlags) 0);
+}
+
+
+
+/*! \brief Set grid_snap_widget text and color
+ *
+ *  \param  widget      This GschemBottomWidget
+ *  \param  str         Text to set. Newly-allocated string. Will be g_free()'d
+ *  \param  color_name  Color name, e.g. "red", "blue"
+ */
+static void
+set_snap_info_widget (GschemBottomWidget* widget,
+                      gchar* str,
+                      const gchar* color_name)
+{
+  g_return_if_fail (widget != NULL);
+
+  set_snap_info_widget_text (widget, str);
+  set_snap_info_widget_color (widget, color_name);
+}
+
+
+
+/*! \brief Reset grid_snap_widget visual state
+ *
+ *  \par Function Description
+ *  Remove any text decorations, reset text color to black
+ *
+ *  \param  widget  This GschemBottomWidget
+ */
+static void
+reset_snap_info_widget (GschemBottomWidget* widget)
+{
+  g_return_if_fail (widget != NULL);
+
+  gchar* txt = g_strdup_printf ("Snap: %d", widget->snap_size);
+  set_snap_info_widget (widget, txt, "black");
+}
+
+
+
+/*! \brief Update info in the grid_snap_widget depending on current snap settings
+ *
+ *  \param  widget  This GschemBottomWidget
+ */
+static void
+update_snap_info_widget (GschemBottomWidget* widget)
+{
+  g_return_if_fail (widget != NULL);
+
+  gchar* cwd = g_get_current_dir();
+  EdaConfig* cfg = eda_config_get_context_for_path (cwd);
+  g_free (cwd);
+
+  gint default_snap_size = DEFAULT_SNAP_SIZE;
+
+  if (cfg != NULL)
+  {
+    GError* err = NULL;
+    gint val = eda_config_get_int (cfg,
+                                   "schematic.gui",
+                                   "snap-size",
+                                   &err);
+    if (err == NULL)
+    {
+      default_snap_size = val;
+    }
+
+    g_clear_error (&err);
   }
 
-  return type;
+
+  reset_snap_info_widget (widget);
+
+  gchar* txt = NULL;
+
+  if (widget->snap_mode == SNAP_OFF)
+  {
+      txt = g_strdup_printf (_("<b>Snap: OFF</b>"));
+      set_snap_info_widget (widget, txt, "blue");
+  }
+  else
+  if (widget->snap_mode == SNAP_GRID)
+  {
+    if (widget->snap_size != default_snap_size)
+    {
+      txt = g_strdup_printf (_("Snap: <b>%d</b>"), widget->snap_size);
+      set_snap_info_widget (widget, txt, "red");
+    }
+  }
+  else
+  if (widget->snap_mode == SNAP_RESNAP)
+  {
+    if (widget->snap_size != default_snap_size)
+    {
+      txt = g_strdup_printf (_("<u>Re</u>snap: <b>%d</b>"), widget->snap_size);
+      set_snap_info_widget(widget, txt, "red");
+    }
+    else
+    {
+      txt = g_strdup_printf (_("<u>Re</u>snap: %d"), widget->snap_size);
+      set_snap_info_widget_text(widget, txt);
+    }
+  }
+  else
+  {
+    g_critical ("%s: update_snap_info_widget(): widget->snap_mode out of range: %d\n",
+                __FILE__,
+                widget->snap_mode);
+  }
+
+
+  if (widget->snap_mode != SNAP_OFF &&
+      widget->snap_size != default_snap_size)
+  {
+    gchar* tooltip = g_strdup_printf(
+      _("Snap size.\n"
+        "Attention: current snap size (%d) differs\n"
+        "from the value set in configuration: %d."),
+        widget->snap_size,
+        default_snap_size);
+
+    gtk_widget_set_tooltip_text (widget->grid_snap_widget,
+                                   tooltip);
+    g_free (tooltip);
+  }
+  else
+  {
+    gtk_widget_set_tooltip_text (widget->grid_snap_widget,
+                                 _("Snap size"));
+  }
+
+} /* update_snap_info_widget() */
+
+
+
+/*! \brief Create grid_snap_widget
+ *
+ *  \param  widget  This GschemBottomWidget
+ */
+static void
+create_snap_info_widget (GschemBottomWidget* widget)
+{
+  g_return_if_fail (widget != NULL);
+
+  widget->grid_snap_widget = gtk_label_new (NULL);
+
+  gchar* str = g_strdup_printf ("Snap: <b>%d</b>", widget->snap_size);
+  gtk_label_set_markup (GTK_LABEL(widget->grid_snap_widget), str);
+  g_free (str);
+
+  gtk_misc_set_padding (GTK_MISC (widget->grid_snap_widget),
+                        LABEL_XPAD,
+                        LABEL_YPAD);
+  gtk_box_pack_start (GTK_BOX (widget), widget->grid_snap_widget, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (widget), gtk_vseparator_new(), FALSE, FALSE, 0);
+}
+
+
+
+/*! \brief Update info in the grid_size_widget depending on current grid settings
+ *
+ *  \param  widget  This GschemBottomWidget
+ */
+static void
+update_grid_size_widget (GschemBottomWidget* widget)
+{
+  g_return_if_fail (widget != NULL);
+
+  gchar* str = NULL;
+
+  if (widget->grid_mode == GRID_MODE_NONE)
+  {
+    str = g_strdup (_("Grid: OFF"));
+  }
+  else
+  if (widget->grid_size <= 0)
+  {
+    str = g_strdup (_("Grid: NONE"));
+  }
+  else
+  {
+    str = g_strdup_printf ("Grid: %d", widget->grid_size);
+  }
+
+  gtk_label_set_label (GTK_LABEL(widget->grid_size_widget), str);
+  gtk_widget_set_tooltip_text (widget->grid_size_widget,
+                               _("Grid size"));
+  g_free (str);
+}
+
+
+
+/*! \brief Create grid_size_widget
+ *
+ *  \param  widget  This GschemBottomWidget
+ */
+static void
+create_grid_size_widget (GschemBottomWidget* widget)
+{
+  g_return_if_fail (widget != NULL);
+
+  widget->grid_size_widget = gtk_label_new (NULL);
+
+  gchar* str = g_strdup_printf ("Grid: %d", widget->grid_size);
+  gtk_label_set_markup (GTK_LABEL(widget->grid_size_widget),
+                        str);
+  g_free (str);
+
+  gtk_misc_set_padding (GTK_MISC (widget->grid_size_widget),
+                        LABEL_XPAD,
+                        LABEL_YPAD);
+  gtk_box_pack_start (GTK_BOX (widget), widget->grid_size_widget, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (widget), gtk_vseparator_new(), FALSE, FALSE, 0);
 }
 
 
@@ -523,8 +751,13 @@ gschem_bottom_widget_init (GschemBottomWidget *widget)
   gboolean show_mouse_indicators       = TRUE;
   gboolean show_rubber_band_indicator  = FALSE;
   gboolean show_magnetic_net_indicator = FALSE;
+#ifdef ENABLE_GTK3
+  gdk_rgba_parse (&widget->status_inactive_color, "black");
+  gdk_rgba_parse (&widget->status_active_color, "green");
+#else
   gdk_color_parse ("black", &widget->status_inactive_color);
   gdk_color_parse ("green", &widget->status_active_color);
+#endif
   widget->status_bold_font = FALSE;
 
   gchar* cwd = g_get_current_dir();
@@ -570,7 +803,11 @@ gschem_bottom_widget_init (GschemBottomWidget *widget)
                                           &err);
     if (color != NULL)
     {
+#ifdef ENABLE_GTK3
+      gdk_rgba_parse (&widget->status_active_color, color);
+#else
       gdk_color_parse (color, &widget->status_active_color);
+#endif
       g_free (color);
     }
     g_clear_error (&err);
@@ -597,10 +834,9 @@ gschem_bottom_widget_init (GschemBottomWidget *widget)
   }
 
 
-  widget->grid_label = gtk_label_new (NULL);
-  gtk_widget_set_tooltip_text (widget->grid_label, _("(Snap size, Grid size)"));
-  gtk_misc_set_padding (GTK_MISC (widget->grid_label), LABEL_XPAD, LABEL_YPAD);
-  gtk_box_pack_start (GTK_BOX (widget), widget->grid_label, FALSE, FALSE, 0);
+  create_snap_info_widget (widget);
+  create_grid_size_widget (widget);
+
 
   separator = gtk_vseparator_new ();
   gtk_box_pack_start (GTK_BOX (widget), separator, FALSE, FALSE, 0);
@@ -797,7 +1033,11 @@ gschem_bottom_widget_set_status_text_color (GschemBottomWidget *widget, gboolean
 {
   g_return_if_fail (widget != NULL);
 
+#ifdef ENABLE_GTK3
+  const GdkRGBA* color = NULL;
+#else
   const GdkColor* color = NULL;
+#endif
 
   if (active) {
     color = &widget->status_active_color;
@@ -805,7 +1045,15 @@ gschem_bottom_widget_set_status_text_color (GschemBottomWidget *widget, gboolean
     color = &widget->status_inactive_color;
   }
 
-  gtk_widget_modify_fg (GTK_WIDGET (widget->status_label), GTK_STATE_NORMAL, color);
+#ifdef ENABLE_GTK3
+  gtk_widget_override_color (GTK_WIDGET (widget->status_label),
+                             GTK_STATE_FLAG_NORMAL,
+                             color);
+#else
+  gtk_widget_modify_fg (GTK_WIDGET (widget->status_label),
+                        GTK_STATE_NORMAL,
+                        color);
+#endif
 }
 
 
@@ -925,7 +1173,7 @@ set_property (GObject *object, guint param_id, const GValue *value, GParamSpec *
 
 
 
-/*! \brief Write the grid settings to the gschem "status bar."
+/*! \brief Write the snap and grid settings to the status bar.
  *
  *  \param [in] widget This GschemBottomWidget
  *  \param [in] pspec  The parameter that changed
@@ -934,46 +1182,8 @@ set_property (GObject *object, guint param_id, const GValue *value, GParamSpec *
 static void
 update_grid_label (GschemBottomWidget *widget, GParamSpec *pspec, gpointer unused)
 {
-  if (widget->grid_label != NULL) {
-    gchar *grid_text = NULL;
-    gchar *label_text = NULL;
-    gchar *snap_text = NULL;
-
-    switch (widget->snap_mode) {
-      case SNAP_OFF:
-        snap_text = g_strdup (_("OFF"));
-        break;
-
-      case SNAP_GRID:
-        snap_text = g_strdup_printf ("%d", widget->snap_size);
-        break;
-
-      case SNAP_RESNAP:
-        snap_text = g_strdup_printf ("%dR", widget->snap_size);
-        break;
-
-      default:
-        g_critical ("%s: update_grid_label(): widget->snap_mode out of range: %d\n", __FILE__, widget->snap_mode);
-    }
-
-    if (widget->grid_mode == GRID_MODE_NONE) {
-      grid_text = g_strdup (_("OFF"));
-    } else {
-      if (widget->grid_size <= 0) {
-        grid_text = g_strdup (_("NONE"));
-      } else {
-        grid_text = g_strdup_printf ("%d", widget->grid_size);
-      }
-    }
-
-    label_text = g_strdup_printf (_("Grid(%1$s, %2$s)"), snap_text, grid_text);
-
-    gtk_label_set_text (GTK_LABEL (widget->grid_label), label_text);
-
-    g_free (grid_text);
-    g_free (label_text);
-    g_free (snap_text);
-  }
+  update_snap_info_widget (widget);
+  update_grid_size_widget (widget);
 }
 
 
@@ -989,12 +1199,19 @@ update_rubber_band_label (GschemBottomWidget *widget, GParamSpec *pspec, gpointe
 {
   g_return_if_fail (widget != NULL);
 
+#ifdef ENABLE_GTK3
+  GdkRGBA color;
+  gdk_rgba_parse (&color, widget->rubber_band_mode ? "green" : "blue");
+  gtk_widget_override_color (GTK_WIDGET (widget->rubber_band_label),
+                             GTK_STATE_FLAG_NORMAL,
+                             &color);
+#else
   GdkColor color;
   gdk_color_parse (widget->rubber_band_mode ? "green" : "blue", &color);
-
   gtk_widget_modify_fg (GTK_WIDGET (widget->rubber_band_label),
                         GTK_STATE_NORMAL,
                         &color);
+#endif
 
   gtk_label_set_markup (GTK_LABEL (widget->rubber_band_label),
                         widget->rubber_band_mode ?
@@ -1014,12 +1231,19 @@ update_magnetic_net_label (GschemBottomWidget *widget, GParamSpec *pspec, gpoint
 {
   g_return_if_fail (widget != NULL);
 
+#ifdef ENABLE_GTK3
+  GdkRGBA color;
+  gdk_rgba_parse (&color, widget->magnetic_net_mode ? "purple" : "darkgray");
+  gtk_widget_override_color (GTK_WIDGET (widget->magnetic_net_label),
+                             GTK_STATE_FLAG_NORMAL,
+                             &color);
+#else
   GdkColor color;
   gdk_color_parse (widget->magnetic_net_mode ? "purple" : "darkgray", &color);
-
   gtk_widget_modify_fg (GTK_WIDGET (widget->magnetic_net_label),
                         GTK_STATE_NORMAL,
                         &color);
+#endif
 
   gtk_label_set_markup (GTK_LABEL (widget->magnetic_net_label),
                     widget->magnetic_net_mode ?
