@@ -220,9 +220,9 @@ void attrib_edit_dialog_ok(GtkWidget * w, GschemToplevel *w_current)
       if (invocation_flag == FROM_HOTKEY
           && wx != -1 && wy != -1) {
         o_invalidate (w_current, new_object);
-        new_object->text->x = wx;
-        new_object->text->y = wy;
-        o_text_recreate (new_object);
+        lepton_text_object_set_x (new_object, wx);
+        lepton_text_object_set_y (new_object, wy);
+        lepton_text_object_recreate (new_object);
     gschem_toplevel_page_content_changed (w_current, toplevel->page_current);
         o_undo_savestate_old(w_current, UNDO_ALL);
       }
@@ -512,17 +512,22 @@ void attrib_edit_dialog (GschemToplevel *w_current, LeptonObject *attr_obj, int
   if (attr_obj) {
     o_attrib_get_name_value (attr_obj, &name, &val);
     attrib = attr_obj;
-    if (lepton_object_is_visible (attrib)) {
+    if (lepton_text_object_is_visible (attrib)) {
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(visbutton), TRUE);
     } else {
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(visbutton), FALSE);
     }
 
-    if (attrib->show_name_value == SHOW_VALUE) {
+    if (lepton_text_object_get_show (attrib) == SHOW_VALUE)
+    {
       gtk_combo_box_set_active (GTK_COMBO_BOX (show_options), 0);
-    } else if (attrib->show_name_value == SHOW_NAME) {
+    }
+    else if (lepton_text_object_get_show (attrib) == SHOW_NAME)
+    {
       gtk_combo_box_set_active (GTK_COMBO_BOX (show_options), 1);
-    } else {
+    }
+    else
+    {
       gtk_combo_box_set_active (GTK_COMBO_BOX (show_options), 2);
     }
   } else {
